#include "lexer.h"
#include "run.h"
#include "sml.h"
#include "buffer.h"

static char lastc;
static Context lastx;

static int
put(char c)
{
    return buffer_PUTC(buffer_1, c);
}

static int
dump_hex(int h)
{
    return put("0123456789abcdef"[h]);
}

static void
dump_char(char c, Context x)
{
    int u = x.flags & U;

    lastc = c;
    lastx = x;

    while (u) {
	put('U');
	u--;
    }
    if (x.flags & B) put('B');
    if (x.flags & EM) put('E');
    if (x.flags & I) put('I');
    if (x.flags & S) put('S');
    if (x.flags & TT) put('T');
    if (x.size) put(x.size);
    if (x.color) put(x.color);

    dump_hex((c >> 4) & 0xf);
    dump_hex(c & 0xf);

    put('\n');
}

static Context
real_space_context(Context x)
{
    Context sx;

    sx.flags = x.flags & ~(S|EM|B|I);
    if (sx.flags & U) sx.color = 'w';
    else sx.color = x.color;
    sx.size = x.size;

    return sx;
}

static void
dump_run_chars(char *p, Run *run)
{
    int i;

    for (i = run->len; i > 0; --i) {
	char c = *p++;
	if (c == ' ' || c == '\t' || c == '\r' || c == '\n') {
	    Context sx = real_space_context(run->x);
	    if (sx.flags & TT) {
		dump_char(c, sx);
	    }

	    else if (1
		&& lastc == ' '
		&& lastx.flags == sx.flags
		&& lastx.size == sx.size
		&& lastx.color == sx.color
	    ) {
		/* do nothing */
	    }

	    else dump_char(' ', sx);
	}

	else dump_char(c, run->x);
    }
}

static void
dump(void)
{
    int i;
    char *p;
    Run *run;

    p = chars.s;
    run = markup.s;
    for (i = 0; i < markup.len; i++) {
	dump_run_chars(p, run);
	p += run->len;
	run++;
    }

    buffer_flush(buffer_1);
}

int
main(int argc, char **argv)
{
    yylex();
    dump();
    return 0;
}

